@@ -29,7 +29,6 @@
 #pod =for :this This is C<Dist::Zilla::Role::ErrorLogger> role documentation. Read this if you want to
 #pod have error logging capabilities in your Dist::Zilla plugin.
 #pod
-#pod
 #pod =head1 SYNOPSIS
 #pod
 #pod     package Dist::Zilla::Plugin::YourPlugin;
@@ -75,10 +74,12 @@ use feature qw{ state };
 use namespace::autoclean;
 
 # ABSTRACT: Have error logging capabilities in your Dist::Zilla plugin
-our $VERSION = '0.005'; # VERSION
+our $VERSION = '0.006'; # VERSION
 
 requires qw{ log log_fatal };
 
+use List::Util qw{ min max };
+
 # --------------------------------------------------------------------------------------------------
 
 #pod =attr error_count
@@ -112,7 +113,7 @@ has error_count => (
 #pod
 #pod =cut
 
-sub log_error {
+sub log_error {                         ## no critic ( RequireArgUnpacking )
     my $self = shift( @_ );
     #   If the first argument is a hashref, it is treated as extra arguments to logging, not as
     #   message, see <https://metacpan.org/pod/Log::Dispatchouli#log>. Let us add `level` argument.
@@ -150,21 +151,21 @@ sub log_error {
 #pod     $self->abort( @items );
 #pod     $self->abort( \%args, @items );
 #pod
-#pod This method logs all the arguments and aborts C<Dist::Zilla>. Does it look similar to C<log_fatal>?
-#pod Yep. This is attempt to workaround L<C<log_fatal>
-#pod drawback|https://github.com/rjbs/Dist-Zilla/issues/397>. In contrast to C<log_fatal>, C<abort>
-#pod guarantees the message appears on the screen only once.
+#pod This is an attempt to workaround L<C<log_fatal>
+#pod drawback|https://github.com/rjbs/Dist-Zilla/issues/397>: in contrast to C<log_fatal>, C<abort>
+#pod guarantees the message (which can be quite long) appears on the screen only once.
 #pod
-#pod TODO: Describe exception.
+#pod The method log the message (via C<log>), then flush C<STDOUT>, then C<die>s with short message
+#pod C<"Aborting...\n">.
 #pod
 #pod =cut
 
-sub abort {
+sub abort {                             ## no critic ( RequireArgUnpacking )
     my $self = shift( @_ );
     if ( @_ ) {
         $self->log( @_ );
     };
-    #~ $self->log_fatal( "Aborting..." );
+    STDOUT->flush();
     die( "Aborting...\n" );
 };
 
@@ -180,11 +181,134 @@ sub abort {
 #pod
 #pod =cut
 
-sub abort_if_error {
+sub abort_if_error {                    ## no critic ( RequireArgUnpacking )
     my $self = shift( @_ );
     if ( $self->error_count ) {
         $self->abort( @_  );
     };
+    return;
+};
+
+# --------------------------------------------------------------------------------------------------
+
+#pod =method log_errors_in_file
+#pod
+#pod The method intended to report errors against a file. It prints file name (and colon after it), then
+#pod prints line-numbered file content annotated by error messages. The method does not print entire
+#pod file content, but only error lines with surrounding context (2 lines above and below each error
+#pod line).
+#pod
+#pod     $self->log_errors_in_file(
+#pod         $file,
+#pod         $linenum0 => $message0,
+#pod         $linenum1 => $message1,
+#pod         ...
+#pod     );
+#pod
+#pod C<$file> should be a C<Dist::Zilla> file (e. g. C<Dist::Zilla::File::OnDisk>,
+#pod C<Dist::Zilla::File::InMemory>, or does role C<Dist::Zilla::Role::File>).
+#pod
+#pod Errors are specified by pairs C<< $linenum => $message >>, where C<$linenum> is a number of problem
+#pod line (one-based), and C<$message> is an error message. Order of errors does not matter usually.
+#pod However, if messages are associated with the same line, they will be printed in order of appearance
+#pod (Yes, a line may have associated more than one error message.)
+#pod
+#pod Zero or negative line numbers, or line numbers beyond the last line are invalid. Messages
+#pod associated with invalid line numbers are reported in unspecified way.
+#pod
+#pod Normally, the method prints all the information by calling C<log_error> method and returns a
+#pod positive integer. However, If any invalid line numbers are specified, the method returns negative
+#pod integer. If no errors are specified, the method prints "No errors found at FILE." by calling
+#pod C<log> (not C<log_error>!) and returns zero.
+#pod
+#pod TODO: Example.
+#pod
+#pod =cut
+
+sub log_errors_in_file {
+
+    my ( $self, $file, @errors ) = @_;
+
+    # Corner case: no errors specified.
+    if ( not @errors ) {
+        $self->log( [ 'No errors at %s.', $file->name ] );
+        return 0;
+    };
+
+    #   TODO: Chop too long lines?
+    my $text  = [ split( "\n", $file->content ) ];
+
+    #   Parse `@errors`.
+    my %errors;     # Key is line number, value is arrayref to error messages.
+    my %invalid;    # The same but for invalid line numbers.
+    while ( @errors ) {
+        my ( $n, $message ) = splice( @errors, 0, 2 );
+        if ( 1<= $n and $n <= @$text ) {
+            my $ctx = 2; # TODO: Parametrize it?
+            for my $k ( max( $n - $ctx, 1 ) .. min( $n + $ctx, @$text + 0 ) ) {
+                if ( not $errors{ $k } ) {
+                    $errors{ $k } = [];
+                };
+            };
+            push( @{ $errors{ $n } }, $message );
+        } else {
+            push( @{ $invalid{ $n } }, $message );
+        };
+    };
+
+    my $t = ' ' x 4;                                # Indent for text lines.
+
+    if ( %errors ) {
+
+        my $w = length( 0 + @$text );               # Width of linenumber column.
+        my $e = $t . ( ' ' x ( $w + 2 ) );          # Indent for error messages.
+        my $last = 0;                               # Number of the last printed text line.
+
+        my $log_line = sub {        # Log text line number and content.
+            my ( $n ) = @_;
+            my $line = $n <= @$text ? $text->[ $n - 1 ] : '';
+            chomp( $line );
+            $self->log_error( [ '%s%0*d: %s', $t, $w, $n, $line ] );
+        };
+        my $log_messages = sub {    # Log error messahes.
+            my ( $n ) = @_;
+            $self->log_error( [ '%s^^^ %s ^^^', $e, $_ ] ) for @{ $errors{ $n } };
+        };
+        my $log_skipped = sub {     # Log number of skipped lines.
+            my ( $n ) = @_;
+            if ( $n > $last + 1 ) {                 # There are skipped lines.
+                my $count = $n - $last - 1;         # Number of skipped lines.
+                if ( $count == 1 ) {
+                    $log_line->( $n - 1 );          # There is no sense to skip one line.
+                } else {
+                    $self->log_error( [ '%s... skipped %d lines ...', $e, $count ] );
+                };
+            };
+        };
+
+        #   Do actual logging.
+        $self->log_error( [ '%s:', $file->name ] );
+        for my $n ( sort( { $a <=> $b } keys( %errors ) ) ) {
+            $log_skipped->( $n );
+            $log_line->( $n );
+            $log_messages->( $n );
+            $last = $n;
+        };
+        $log_skipped->( @$text + 1 );
+
+    };
+
+    if ( %invalid ) {
+        $self->log_error( 'Following errors are reported against non-existing lines of the file:' );
+        for my $n ( sort( { $a <=> $b } keys( %invalid ) ) ) {
+            $self->log_error( [ '%s%s at %s line %d.', $t, $_, $file->name, $n ] )
+                for @{ $invalid{ $n } };
+        };
+        return -1;
+    };
+
+    return 1;
+
 };
 
 # --------------------------------------------------------------------------------------------------
@@ -219,11 +343,12 @@ sub abort_if_error {
 #pod
 #pod =head1 SEE ALSO
 #pod
-#pod L<Dist::Zilla>
-#pod L<Dist::Zilla::Role>
-#pod L<Dist::Zilla::Plugin>
-#pod L<Log::Dispatchouli>
-#pod L<String::Flogger>
+#pod =for :list
+#pod = L<Dist::Zilla>
+#pod = L<Dist::Zilla::Role>
+#pod = L<Dist::Zilla::Plugin>
+#pod = L<Log::Dispatchouli>
+#pod = L<String::Flogger>
 #pod
 #pod =cut
 
@@ -234,7 +359,7 @@ sub abort_if_error {
 #pod =head1 WHAT?
 #pod
 #pod C<Dist-Zilla-Role-ErrorLogger> is a C<Dist::Zilla> role. It provides C<log_error>, C<abort>, and
-#pod C<abort_if_error> methods to consuming plugin.
+#pod C<abort_if_error> methods to consuming plugins.
 #pod
 #pod =cut
 
@@ -305,12 +430,12 @@ Dist::Zilla::Role::ErrorLogger - Have error logging capabilities in your Dist::Z
 
 =head1 VERSION
 
-Version 0.005, released on 2015-07-14 19:16 UTC.
+Version 0.006, released on 2015-09-16 17:19 UTC.
 
 =head1 WHAT?
 
 C<Dist-Zilla-Role-ErrorLogger> is a C<Dist::Zilla> role. It provides C<log_error>, C<abort>, and
-C<abort_if_error> methods to consuming plugin.
+C<abort_if_error> methods to consuming plugins.
 
 This is C<Dist::Zilla::Role::ErrorLogger> role documentation. Read this if you want to
 have error logging capabilities in your Dist::Zilla plugin.
@@ -372,12 +497,12 @@ attribute. The method returns true value, so can be used in following constructs
     $self->abort( @items );
     $self->abort( \%args, @items );
 
-This method logs all the arguments and aborts C<Dist::Zilla>. Does it look similar to C<log_fatal>?
-Yep. This is attempt to workaround L<C<log_fatal>
-drawback|https://github.com/rjbs/Dist-Zilla/issues/397>. In contrast to C<log_fatal>, C<abort>
-guarantees the message appears on the screen only once.
+This is an attempt to workaround L<C<log_fatal>
+drawback|https://github.com/rjbs/Dist-Zilla/issues/397>: in contrast to C<log_fatal>, C<abort>
+guarantees the message (which can be quite long) appears on the screen only once.
 
-TODO: Describe exception.
+The method log the message (via C<log>), then flush C<STDOUT>, then C<die>s with short message
+C<"Aborting...\n">.
 
 =head2 abort_if_error
 
@@ -387,29 +512,37 @@ TODO: Describe exception.
 If there was any errors (i. e. C<error_count> is greater than zero), the logs all the arguments and
 aborts execution. Both actions (logging and aborting) are implemented by calling C<abort>.
 
-=head1 NOTES
+=head2 log_errors_in_file
 
-All the methods defined in the role log items through the C<log> method. C<Dist::Zilla> takes this
-method from C<Log::Dispatchouli>, the latter uses C<String::Flogger> to process the messages. It
-means you can use C<String::Flogger> tricks, e. g.:
+The method intended to report errors against a file. It prints file name (and colon after it), then
+prints line-numbered file content annotated by error messages. The method does not print entire
+file content, but only error lines with surrounding context (2 lines above and below each error
+line).
 
-    $self->log_error( [ 'oops at %s line %d', $file, $line ] );
-        #   [] are shorter than sprintf.
+    $self->log_errors_in_file(
+        $file,
+        $linenum0 => $message0,
+        $linenum1 => $message1,
+        ...
+    );
 
-Also note how C<Log::Dispatchouli> describes the C<log> method:
+C<$file> should be a C<Dist::Zilla> file (e. g. C<Dist::Zilla::File::OnDisk>,
+C<Dist::Zilla::File::InMemory>, or does role C<Dist::Zilla::Role::File>).
 
-    $logger->log( @messages );
+Errors are specified by pairs C<< $linenum => $message >>, where C<$linenum> is a number of problem
+line (one-based), and C<$message> is an error message. Order of errors does not matter usually.
+However, if messages are associated with the same line, they will be printed in order of appearance
+(Yes, a line may have associated more than one error message.)
 
-and says:
+Zero or negative line numbers, or line numbers beyond the last line are invalid. Messages
+associated with invalid line numbers are reported in unspecified way.
 
-    Each message is flogged individually, then joined with spaces.
+Normally, the method prints all the information by calling C<log_error> method and returns a
+positive integer. However, If any invalid line numbers are specified, the method returns negative
+integer. If no errors are specified, the method prints "No errors found at FILE." by calling
+C<log> (not C<log_error>!) and returns zero.
 
-So beware. A call
-
-    $self->log_error( 'error 1', 'error 2' );
-
-logs I<one message> "error 1 error 2", I<not> two messages "error 1" and "error 2", and bumps
-C<error_count> by 1, not 2.
+TODO: Example.
 
 =head1 WHY?
 
@@ -454,13 +587,45 @@ C<Dist-Zilla-Role-ErrorLogger> role automates it, making plugin code shorter and
     …
     $self->abort_if_error();
 
+=head1 NOTES
+
+All the methods defined in the role log items through the C<log> method. C<Dist::Zilla> takes this
+method from C<Log::Dispatchouli>, the latter uses C<String::Flogger> to process the messages. It
+means you can use C<String::Flogger> tricks, e. g.:
+
+    $self->log_error( [ 'oops at %s line %d', $file, $line ] );
+        #   [] are shorter than sprintf.
+
+Also note how C<Log::Dispatchouli> describes the C<log> method:
+
+    $logger->log( @messages );
+
+and says:
+
+    Each message is flogged individually, then joined with spaces.
+
+So beware. A call
+
+    $self->log_error( 'error 1', 'error 2' );
+
+logs I<one message> "error 1 error 2", I<not> two messages "error 1" and "error 2", and bumps
+C<error_count> by 1, not 2.
+
 =head1 SEE ALSO
 
-L<Dist::Zilla>
-L<Dist::Zilla::Role>
-L<Dist::Zilla::Plugin>
-L<Log::Dispatchouli>
-L<String::Flogger>
+=over 4
+
+=item L<Dist::Zilla>
+
+=item L<Dist::Zilla::Role>
+
+=item L<Dist::Zilla::Plugin>
+
+=item L<Log::Dispatchouli>
+
+=item L<String::Flogger>
+
+=back
 
 =head1 AUTHOR
 
